"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.exportSite = void 0;
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const consola_1 = __importDefault(require("consola"));
const chalk_1 = __importDefault(require("chalk"));
const serialize_javascript_1 = __importDefault(require("serialize-javascript"));
const server_1 = require("@ream/server");
const promise_queue_1 = require("@egoist/promise-queue");
const flatten_routes_1 = require("./utils/flatten-routes");
function getHref(attrs) {
    const match = /href\s*=\s*(?:"(.*?)"|'(.*?)'|([^\s>]*))/.exec(attrs);
    return match && (match[1] || match[2] || match[3]);
}
const exportSite = async (dotReamDir, fullyExport) => {
    const exportDir = path_1.default.join(dotReamDir, fullyExport ? 'client' : 'export');
    const { serverContext } = require(path_1.default.join(dotReamDir, 'meta/server-context'));
    const globalPreload = await serverContext.serverEntry.getGlobalPreload();
    // Adding a global `preload` function in `_app.vue` will disable automatic static generation
    if (globalPreload && !fullyExport)
        return;
    const clientRoutes = await flatten_routes_1.flattenRoutes(serverContext.serverEntry.clientRoutes);
    const staticPaths = [];
    const exportManifest = { staticPages: [] };
    await Promise.all(clientRoutes.map(async (route) => {
        const shouldExport = route.matched.every((m) => !m.preload);
        if (!shouldExport) {
            return;
        }
        if (route.path === '/:404(.*)') {
            staticPaths.push('/404.html');
        }
        else {
            const isDynamicPath = route.path.includes(':');
            let fallback = false;
            const isStatic = route.matched.every((m) => !m.preload);
            if (!isStatic)
                return;
            if (isDynamicPath) {
                const paths = [];
                for (const m of route.matched) {
                    if (m.getStaticPaths) {
                        const res = await m.getStaticPaths();
                        paths.push(...res.paths);
                        if (res.fallback != null) {
                            fallback = res.fallback;
                        }
                    }
                }
                if (paths.length === 0) {
                    consola_1.default.warn(`No static paths provided for ${route.path}, skipped exporting`);
                }
                for (const path of paths) {
                    const staticPath = route.path
                        .replace('(.*)', '')
                        .replace(/:([^\/]+)/g, (_, p1) => path.params[p1]);
                    staticPaths.push(staticPath);
                }
            }
            else {
                staticPaths.push(route.path);
            }
            exportManifest.staticPages.push({ path: route.path, fallback });
        }
    }));
    await fs_extra_1.default.outputFile(path_1.default.join(dotReamDir, 'manifest/export-manifest.json'), JSON.stringify(exportManifest), 'utf8');
    const queue = new promise_queue_1.PromiseQueue(async (jobId, url) => {
        consola_1.default.info(chalk_1.default.dim(jobId));
        const router = await server_1.createClientRouter(serverContext.serverEntry, url);
        const routePath = router.currentRoute.value.path;
        const { html = '', preloadResult } = await server_1.render({
            url,
            ssrManifest: serverContext.ssrManifest,
            serverEntry: serverContext.serverEntry,
            clientManifest: serverContext.clientManifest,
            getHtmlAssets: server_1.productionGetHtmlAssets,
            router,
        });
        const htmlPath = server_1.getExportOutputPath(routePath, 'html', exportDir);
        const jsonPath = server_1.getExportOutputPath(routePath, 'json', exportDir);
        await Promise.all([
            fs_extra_1.default.outputFile(htmlPath, html, 'utf8'),
            preloadResult.hasPreload &&
                preloadResult.isStatic &&
                (await fs_extra_1.default.outputFile(jsonPath, serialize_javascript_1.default(preloadResult, { isJSON: true }), 'utf8')),
        ]);
        // Crawl pages
        if (fullyExport) {
            // find all `<a>` tags in exported html files and export links that are not yet exported
            let match = null;
            const LINK_RE = /<a ([\s\S]+?)>/gm;
            while ((match = LINK_RE.exec(html))) {
                const href = getHref(match[1]);
                if (href) {
                    const url = new URL(href, 'http://self');
                    if (url.host === 'self') {
                        queue.add(`Exporting ${url.pathname}`, url.pathname);
                    }
                }
            }
        }
    }, { maxConcurrent: 100 });
    for (const path of staticPaths) {
        queue.add(`Exporting ${path}`, path);
    }
    await queue.run();
    if (fullyExport) {
        consola_1.default.success(`Your app has been exported to ${path_1.default.relative(process.cwd(), path_1.default.join(dotReamDir, 'client'))}`);
    }
};
exports.exportSite = exportSite;
