"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getViteConfig = void 0;
const path_1 = __importDefault(require("path"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const plugin_vue_1 = __importDefault(require("@vitejs/plugin-vue"));
const babel_1 = require("./plugins/babel");
const reamAliasPlugin = (api) => {
    return {
        name: `ream:alias`,
        resolveId(source) {
            // Bundle @ream/app since it's written in esnext modules
            // Otherwise it will break in Node.js (SSR)
            if (source === '@ream/app' || source.startsWith('@ream/app/')) {
                return require.resolve(source);
            }
            return undefined;
        },
    };
};
// TODO: see if it can be fixed over https://github.com/vitejs/vite/issues/2274
const reamForceServerUpdatePlugin = (api) => {
    return {
        name: `ream:force-server-update`,
        handleHotUpdate(ctx) {
            const { moduleGraph } = api.viteDevServer;
            for (const [key, value] of moduleGraph.fileToModulesMap.entries()) {
                if (!key.includes('node_modules')) {
                    for (const mod of value) {
                        moduleGraph.invalidateModule(mod);
                    }
                }
            }
        },
    };
};
const moveManifestPlugin = (manifestDir) => {
    return {
        name: 'ream:move-manifest',
        async writeBundle(options, bundle) {
            const files = [];
            for (const file in bundle) {
                const value = bundle[file];
                if (file === 'manifest.json' ||
                    (file === 'ssr-manifest.json' && value.type === 'asset')) {
                    files.push({
                        name: file === 'manifest.json' ? 'client-manifest.json' : file,
                        originalPath: path_1.default.join(options.dir, file),
                        // @ts-ignore
                        content: value.source,
                    });
                    delete bundle[file];
                }
            }
            if (files.length > 0) {
                await Promise.all(files.map(async (file) => {
                    await fs_extra_1.default.remove(file.originalPath);
                    await fs_extra_1.default.outputFile(path_1.default.join(manifestDir, file.name), file.content, 'utf8');
                }));
            }
        },
    };
};
const getViteConfig = (api, server) => {
    var _a;
    const ssrManifest = !server && !api.isDev;
    const entry = api.isDev
        ? undefined
        : require.resolve(`@ream/app/${server ? 'server-entry.js' : 'client-entry.js'}`);
    const config = {
        mode: api.mode,
        root: api.rootDir,
        plugins: [
            reamAliasPlugin(api),
            reamForceServerUpdatePlugin(api),
            plugin_vue_1.default({
                include: [/\.vue$/],
            }),
            babel_1.babelPlugin(),
            moveManifestPlugin(api.resolveDotReam('manifest')),
        ],
        define: api.constants,
        resolve: {
            alias: {
                '@': api.resolveSrcDir(),
                vue: ((_a = api.config.vue) === null || _a === void 0 ? void 0 : _a.runtimeTemplateCompiler)
                    ? 'vue/dist/vue.esm-bundler.js'
                    : 'vue/dist/vue.runtime.esm-bundler.js',
            },
        },
        optimizeDeps: {
            // Don't let Vite optimize these deps with esbuild
            exclude: ['@ream/app', '@ream/fetch', 'node-fetch'],
            include: ['vue', 'vue-router', '@vueuse/head', 'mitt'],
        },
        // @ts-expect-error vite does not expose these experimental stuff in types yet
        ssr: {
            // https://vitejs.dev/config/#ssr-external
            external: [
                'vue',
                'vue/dist/vue.esm-bundler.js',
                'vue/dist/vue.runtime.esm-bundler.js',
                'vue-router',
            ],
            noExternal: ['@ream/app'],
        },
        server: {
            middlewareMode: true,
            hmr: api.config.hmr && {
                host: api.config.hmr.host,
                port: api.config.hmr.port,
            },
        },
        build: {
            minify: !api.isDev && !process.env.DEBUG,
            outDir: server
                ? api.resolveDotReam('server')
                : api.resolveDotReam('client'),
            ssrManifest,
            ssr: server,
            manifest: !api.isDev && !server,
            rollupOptions: entry
                ? {
                    input: entry,
                }
                : {},
        },
    };
    if (api.config.vite) {
        api.config.vite(config, { dev: api.isDev, ssr: server });
    }
    return config;
};
exports.getViteConfig = getViteConfig;
